locals {}

data "aws_availability_zones" "available_azs" {
  state = "available"
}

resource "aws_dax_subnet_group" "subnet_group" {
  name       = var.cluster_name
  subnet_ids = var.subnet_ids
}

resource "aws_dax_parameter_group" "parameter_group" {
  name = var.cluster_name

  parameters {
    name  = "query-ttl-millis"
    value = var.query_ttl
  }

  parameters {
    name  = "record-ttl-millis"
    value = var.record_ttl
  }
}

resource "aws_dax_cluster" "dax" {
  cluster_endpoint_encryption_type = var.encryption_type
  cluster_name                     = var.cluster_name
  iam_role_arn                     = var.dax_iam_role_arn
  node_type                        = var.node_type
  replication_factor               = var.replication_factor
  availability_zones               = length(var.availability_zones) == 0 ? var.availability_zones : data.aws_availability_zones.available_azs.names
  description                      = var.description
  notification_topic_arn           = var.notification_topic_arn
  parameter_group_name             = aws_dax_parameter_group.parameter_group.name
  maintenance_window               = var.maintenance_window
  security_group_ids               = var.security_group_ids
  subnet_group_name                = aws_dax_subnet_group.subnet_group.name
  tags                             = var.tags

  server_side_encryption {
    enabled = var.server_side_encryption
  }
}

resource "aws_iam_policy" "read-only-policy" {
  count       = var.enable_iam_policies ? 1 : 0
  name        = "${var.iam_prefix}-dax-read-only"
  path        = "/"
  description = "DAX read-only policy"

  policy = <<EOF
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Action": [
	 "dax:GetItem",
         "dax:BatchGetItem",
         "dax:Query",
         "dax:Scan",
	 "dax:ConditionCheckItem"
      ],
      "Effect": "Allow",
      "Resource": "${aws_dax_cluster.dax.arn}"
    }
  ]
}
EOF
  lifecycle {
    ignore_changes = [
      description,
    ]
  }
  tags = var.tags
}

resource "aws_iam_policy" "read-write-policy" {
  count       = var.enable_iam_policies ? 1 : 0
  name        = "${var.iam_prefix}-dax-read-write"
  path        = "/"
  description = "DAX read/write policy"

  policy = <<EOF
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Action": [
         "dax:GetItem",
         "dax:BatchGetItem",
         "dax:Query",
         "dax:Scan",
         "dax:PutItem",
         "dax:UpdateItem",
         "dax:DeleteItem",
         "dax:BatchWriteItem",
         "dax:ConditionCheckItem"
      ],
      "Effect": "Allow",
      "Resource": "${aws_dax_cluster.dax.arn}"
    }
  ]
}
EOF
  lifecycle {
    ignore_changes = [
      description,
    ]
  }
  tags = var.tags
}
